/* @(#)fheader.c	19.1.1.1 (ESO-DMD) 02/25/03 14:01:58 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT    (c)  1988   European Southern Observatory
.IDENT        fheader.c
.LANGUAGE     C
.AUTHOR       K. Banse   ESO/IPG
.KEYWORDS     FITS header
.PURPOSE      isolate FITS primary header in an ascii file
.VERSION      1.00  021115:   Creation 
 021115		last modif

---------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#include   <fileexts.h>
#include <fsydef.h>

#define MAXREC 2880
/*

*/

int main(argc,argv)
int argc;
char *argv[];

{
int   inMidas, stat, nval, uni, linco, extco;

char  tbuf[120], ffile[120], txtfile[120];



if (argc > 1)				/* we're with command line args. */
   {
   SCSPRO("-1");
   (void) strcpy(ffile,argv[1]);
   inMidas = 0;
   }
else
   {
   SCSPRO("header");
   stat = SCKGETC("IN_A",1,80,&nval,ffile);	
   inMidas = 1;
   uni = 0;
   }

/* pass FITS file name to get_fhead */

stat = get_fhead(ffile,txtfile,&linco,&extco);
if (stat != 0)
   (void) sprintf(tbuf,"extracting FITS header from %s failed",ffile);
else
   {
   (void) sprintf(tbuf,"FITS header stored in %s",txtfile);
   SCTPUT(tbuf);
   (void) sprintf(tbuf,
      "%d extensions, %d lines of FITS keywords in FITS header(s)",extco,linco);
   }
SCTPUT(tbuf);


/* inside Midas we save some info for the wrapper procedure (fheader.prg) */

if (inMidas == 1)
   {
   (void) SCKWRI("OUTPUTI",&stat,1,1,&uni);
   nval = (int) strlen(txtfile);
   if (nval < 80) 
      {
      (void) strcat(txtfile,"                    ");
      nval += 20;
      }
   (void) SCKWRC("OUT_A",1,txtfile,1,nval+20,&uni);
   }

SCSEPI();
}
/*

*/
int get_fhead(fitsfile,txtfile,ico,eco)
char  *fitsfile;
char  *txtfile;
int   *ico;		/* OUT: no. of lines copied */
int   *eco;		/* OUT: no. of extensions in FITS file */

{
int  n, m, count, retstat, endy, ibuf[5];
int  td, fd, nbytes;

char  myrec[MAXREC+2],endmask[24], ascbuf[88];
char  *myptr;



m = CGN_JNDEXC(fitsfile,FSY_TYPMARK);
if (m > 0)
   {
   fitsfile[m] = '\0';
   (void) strcpy(txtfile,fitsfile);
   (void) strcat(txtfile,".fhead");
   fitsfile[m] = FSY_TYPMARK;
   }
else
   {
   (void) strcpy(txtfile,fitsfile);
   (void) strcat(txtfile,".fhead");
   }

*ico = *eco = 0;

fd = open(fitsfile,O_RDONLY);
if (fd == -1)
   {
   (void) printf("could not access %s\n",fitsfile);
   retstat = 1;
   goto byebye;
   }

td = osaopen(txtfile,1);		/* create ascii file */
if (td < 0)
   {
   (void) printf("could not create %s ...\n",txtfile);
   retstat = 2;
   goto byebye;
   }

(void) strcpy(endmask,"END");
endy = -1;				/* end of reading flag */
retstat = 0;


/* FITS records are usually 2880 bytes long (MAXREC set to that value) */

read_loop:
nbytes = read(fd,(void *)myrec,(size_t)MAXREC);
if (nbytes < 0) goto byebye;			/* EOF reached */


/* at start, check if really FITS format */

if (endy == -1)
   {						/* we do this only once */
   if (strncmp(myrec,"SIMPLE",6) != 0)
      {
      (void) printf("not a FITS file ...\n");
      retstat = 4;
      goto byebye;
      }
   endy = 0;
   *eco = 1;
   }

myptr = myrec;				/* point to header buffer */

copy_header:
count = 0;
while (count < nbytes)
   {					/* copy records of 80 bytes */
   (void) strncpy(ascbuf,myptr,80);
   ascbuf[80] = '\0';
   if (strncmp(ascbuf,"END",3) == 0)	/* check for end of header */
      {
      endy = 1;
      ascbuf[3] = '\0';
      }
   n = (int)strlen(ascbuf);
   m = osawrite(td,ascbuf,n);
   if ((m-1) != n)
      {
      (void) printf("problems in writing FITS header to text file ...\n");
      retstat = 5;
      goto byebye;
      }
   (*ico) ++;
   myptr += 80;
   count += 80;
   if (endy == 1) 
      {
      myptr = myrec+count;		/* point to line after "END" */
      nbytes -= count;
      endy = 0;
      if (nbytes > 0) goto xtens_check;		/* still stuff in buffer */
      }
   }
goto read_loop;


/* here we look for the extensions */

xtens_check:
count = 0;			/* check for FITS extensions */
while (count < nbytes)
   {
   if (strncmp(myptr,"XTENSION=",9) == 0)
      {						/* mark it visually */
      char trenn[24];

      (void) strcpy(trenn,"         ");
      (void) osawrite(td,trenn,(int)strlen(trenn));
      (void) strcpy(trenn,"--------------------");
      (void) osawrite(td,trenn,(int)strlen(trenn));
      (void) strcpy(trenn,"         ");
      (void) osawrite(td,trenn,(int)strlen(trenn));
      nbytes -= count;
      (*eco) ++;				/* update extension count */
      goto copy_header;
      }
   myptr += 80;
   count += 80;
   }

nbytes = read(fd,(void *)myrec,(size_t)MAXREC);
if (nbytes > 0) 
   {
   myptr = myrec;
   goto xtens_check;
   }


byebye:
(void) close(fd);
(void) osaclose(td);
return retstat;
}

